@@ -1,4 +1,6 @@
 # Import local modules
+from rezbuild.builder import CompileBuilder
+from rezbuild.builder import ExtractBuilder
 from rezbuild.builder import InstallBuilder
 from rezbuild.builder import PythonBuilder
 from rezbuild.builder import PythonSourceBuilder
@@ -7,6 +9,8 @@ from rezbuild.builder import RezBuilder
 
 
 __all__ = [
+    "CompileBuilder",
+    "ExtractBuilder",
     "InstallBuilder",
     "PythonBuilder",
     "PythonSourceBuilder",
